package cs108;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.junit.Test;

public class ListsTest {
    @Test
    public void testAddMany() {
        List<Number> l = new ArrayList<>();
        Lists.<Double>addMany(l, 3.14, 0); // 1
        assertTrue(l.isEmpty());

        Lists.<Integer>addMany(l, 12, 1); // 2
        assertEquals(1, l.size());
        assertEquals(12, l.get(0));

        Lists.<Double>addMany(l, 0.5, 19); // 3
        assertEquals(20, l.size());

        Iterator<Number> it = l.iterator();
        assertEquals(12, it.next());
        while (it.hasNext())
            assertEquals(0.5, it.next());

        // Invalide: on tente d'ajouter des chaînes à une liste de nombres.
        //      Lists.addMany(l, "55", 1); // 4
        //        assertEquals(55, l.get(20));
    }

    @Test
    public void testCopy() {
        List<Number> ln = new ArrayList<>();
        List<Number> ln2 = new ArrayList<>();
        List<Integer> li = new ArrayList<>();
        List<Double> ld = new ArrayList<>();

        Lists.<Integer>addMany(li, 1, 5);
        Lists.<Double>addMany(ld, 3.14, 5);

        Lists.copy(li, ln); // 5
        assertEquals(Arrays.asList(1,1,1,1,1), ln);
        Lists.copy(ld, ln); // 6
        assertEquals(Arrays.asList(1,1,1,1,1,3.14,3.14,3.14,3.14,3.14), ln);

        Lists.<Number>copy(li, ln2); // 7
        Lists.<Integer>copy(li, ln2); // 8
        assertEquals(Arrays.asList(1,1,1,1,1,1,1,1,1,1), ln2);

        // Invalide: on tente de copier une liste de nombres (qui pourrait
        // p.ex. contenir des doubles) dans une liste d'entiers.
        //        Lists.copy(ln, li); // 9
        //        assertEquals(Arrays.asList(1,1,1,1,1,1,1,1,1,1,3.14,3.14,3.14,3.14,3.14), li);

        // Invalide: on tente de copier une liste de nombres (qui pourrait
        // p.ex. contenir des entiers) dans une liste de doubles.
        //        Lists.copy(ln, ld); // 10
        //        assertEquals(Arrays.asList(3.14,3.14,3.14,3.14,3.14,1,1,1,1,1,3.14,3.14,3.14,3.14,3.14), ld);
    }

    @Test
    public void testMax() {
        Comparator<Number> numComparator = new Comparator<Number>() {
            @Override
            public int compare(Number n1, Number n2) {
                return Double.compare(n1.doubleValue(), n2.doubleValue());
            }
        };
        List<Integer> li = Arrays.asList(1, 2, 3);
        int max = Lists.max(li, numComparator); // 11
        assertEquals(3, max);

        Number max2 = Lists.<Number>max(li, numComparator); // 12
        assertEquals(3, max2);

        // Invalide: étant donné que l'on force T à être Object, le comparateur
        // (qui ne sait que comparer des nombres) ne peut être utilisé ici.
        //        Object max3 = Lists.<Object>max(li, numComparator); // 13
        //        assertEquals(3, max3);
    }
}
